@@ -15,7 +15,7 @@
 import argparse
 
 def add_subparser(subparsers):
-    _parser = subparsers.add_parser("master", description="QuantRocket securities master CLI", help="quantrocket master -h")
+    _parser = subparsers.add_parser("master", description="QuantRocket securities master CLI", help="Manage and query the securities master database")
     _subparsers = _parser.add_subparsers(title="subcommands", dest="subcommand")
     _subparsers.required = True
 
@@ -530,3 +530,218 @@ Delist a security by symbol + exchange:
         choices=["STK", "ETF", "FUT", "CASH", "IND"],
         help="the security type of the security to be delisted (if needed to disambiguate). Possible choices: %(choices)s")
     parser.set_defaults(func="quantrocket.master._cli_delist_security")
+
+    examples = """
+Fetch upcoming trading hours for exchanges and save to securites master database.
+
+Examples:
+
+Fetch trading hours for all exchanges in securities master database:
+
+    quantrocket master fetch-calendar
+    """
+    parser = _subparsers.add_parser(
+        "fetch-calendar",
+        help="fetch upcoming trading hours for exchanges and save to securites master database",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "-e", "--exchanges",
+        help="limit to these exchanges")
+    parser.set_defaults(func="quantrocket.master._cli_fetch_calendar")
+
+    examples = """
+Check whether exchanges are open or closed.
+
+Examples:
+
+Check whether NYSE is open or closed now:
+
+    quantrocket master calendar NYSE
+
+Check whether the Tokyo Stock Exchange was open or closed 5 hours ago:
+
+    quantrocket master calendar TSEJ --ago 5h
+
+Check whether GLOBEX will be open or closed in 30 minutes:
+
+    quantrocket master calendar GLOBEX --in 30min
+    """
+    parser = _subparsers.add_parser(
+        "calendar",
+        help="check whether exchanges are open or closed",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "exchanges",
+        metavar="EXCHANGE",
+        nargs="+",
+        help="the exchange(s) to check")
+    parser.add_argument(
+        "-t", "--sec-type",
+        metavar="SEC_TYPE",
+        choices=["STK", "FUT", "CASH", "OPT"],
+        help="the security type, if needed to disambiguate for exchanges that "
+        "trade multiple security types. Possible choices: %(choices)s")
+    timedelta_group = parser.add_mutually_exclusive_group()
+    timedelta_group.add_argument(
+        "-i", "--in",
+        metavar="TIMEDELTA",
+        dest="in_",
+        help="check whether exchanges will be open or closed at this point in the "
+        "future (use Pandas timedelta string, e.g. 2h or 30min or 1d)")
+    timedelta_group.add_argument(
+        "-a", "--ago",
+        metavar="TIMEDELTA",
+        help="check whether exchanges were open or closed this long ago "
+        "(use Pandas timedelta string, e.g. 2h or 30min or 1d)")
+    parser.set_defaults(func="quantrocket.master._cli_list_calendar_statuses")
+
+    examples = """
+Assert that one or more exchanges are open and exit non-zero if closed.
+
+Intended to be used as a conditional for running other commands.
+
+Examples:
+
+Place Moonshot orders if NYSE is open now:
+
+    quantrocket master isopen NYSE && quantrocket moonshot orders my-strategy | quantrocket blotter order -f -
+
+Fetch historical data for Australian stocks if the exchange was open 4 hours ago:
+
+    quantrocket master isopen ASX --ago 4h && quantrocket history fetch asx-stk-1d
+
+Log a message if the London Stock Exchange will be open in 30 minutes:
+
+    quantrocket master isopen LSE --in 30min && quantrocket flightlog log 'the market opens soon!'
+    """
+    parser = _subparsers.add_parser(
+        "isopen",
+        help="assert that one or more exchanges are open and exit non-zero if closed",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "exchanges",
+        metavar="EXCHANGE",
+        nargs="+",
+        help="the exchange(s) to check")
+    parser.add_argument(
+        "-t", "--sec-type",
+        metavar="SEC_TYPE",
+        choices=["STK", "FUT", "CASH", "OPT"],
+        help="the security type, if needed to disambiguate for exchanges that "
+        "trade multiple security types. Possible choices: %(choices)s")
+    timedelta_group = parser.add_mutually_exclusive_group()
+    timedelta_group.add_argument(
+        "-i", "--in",
+        metavar="TIMEDELTA",
+        dest="in_",
+        help="assert that exchanges will be open at this point in the "
+        "future (use Pandas timedelta string, e.g. 2h or 30min or 1d)")
+    timedelta_group.add_argument(
+        "-a", "--ago",
+        metavar="TIMEDELTA",
+        help="assert that exchanges were open this long ago "
+        "(use Pandas timedelta string, e.g. 2h or 30min or 1d)")
+    parser.set_defaults(func="quantrocket.master._cli_isopen")
+
+    examples = """
+Assert that one or more exchanges are closed and exit non-zero if open.
+
+Intended to be used as a conditional for running other commands.
+
+Examples:
+
+Place Moonshot orders if the NYSE will be closed NYSE in 1 hour:
+
+    quantrocket master isclosed NYSE --in 1h && quantrocket moonshot orders my-strategy | quantrocket blotter order -f -
+
+Fetch historical data for Australian stocks if the exchange is closed now:
+
+    quantrocket master isclosed ASX && quantrocket history fetch asx-stk-1d
+    """
+    parser = _subparsers.add_parser(
+        "isclosed",
+        help="assert that one or more exchanges are closed and exit non-zero if open",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "exchanges",
+        metavar="EXCHANGE",
+        nargs="+",
+        help="the exchange(s) to check")
+    parser.add_argument(
+        "-t", "--sec-type",
+        metavar="SEC_TYPE",
+        choices=["STK", "FUT", "CASH", "OPT"],
+        help="the security type, if needed to disambiguate for exchanges that "
+        "trade multiple security types. Possible choices: %(choices)s")
+    timedelta_group = parser.add_mutually_exclusive_group()
+    timedelta_group.add_argument(
+        "-i", "--in",
+        metavar="TIMEDELTA",
+        dest="in_",
+        help="assert that exchanges will be closed at this point in the "
+        "future (use Pandas timedelta string, e.g. 2h or 30min or 1d)")
+    timedelta_group.add_argument(
+        "-a", "--ago",
+        metavar="TIMEDELTA",
+        help="assert that exchanges were closed this long ago "
+        "(use Pandas timedelta string, e.g. 2h or 30min or 1d)")
+    parser.set_defaults(func="quantrocket.master._cli_isclosed")
+
+    examples = """
+Round prices in a CSV file to valid tick sizes.
+
+CSV should contain columns `ConId`, `Exchange`, and the columns to be rounded
+(e.g. `LmtPrice`). Additional columns will be ignored and returned unchanged.
+
+Examples:
+
+Round the LmtPrice column in a CSV of orders and return a new CSV:
+
+    quantrocket master ticksize -f orders.csv --round LmtPrice -o rounded_orders.csv
+
+Round the StopPrice column in a CSV of orders and append the tick size as a
+new column (called StopPriceTickSize):
+
+    quantrocket master ticksize -f orders.csv -r StopPrice --append-ticksize -o rounded_orders.csv
+
+Round the LmtPrice column in a CSV of Moonshot orders then place the orders:
+
+    quantrocket moonshot orders umd-japan | quantrocket master ticksize -f - -r LmtPrice | quantrocket blotter order -f -
+    """
+    parser = _subparsers.add_parser(
+        "ticksize",
+        help="round prices in a CSV to valid tick sizes",
+        epilog=examples,
+        formatter_class=argparse.RawDescriptionHelpFormatter)
+    parser.add_argument(
+        "-f", "--infile",
+        required=True,
+        metavar="INFILE",
+        dest="infilepath_or_buffer",
+        help="CSV file with prices to be rounded (specify '-' to read file from stdin)")
+    parser.add_argument(
+        "-r", "--round",
+        nargs="+",
+        metavar="FIELD",
+        dest="round_fields",
+        help="columns to be rounded")
+    parser.add_argument(
+        "-d", "--how",
+        metavar="DIRECTION",
+        choices=["up", "down", "nearest"],
+        help="which direction to round to. Possible choices: up, down, nearest "
+        "(default is 'nearest')")
+    parser.add_argument(
+        "-a", "--append-ticksize",
+        action="store_true",
+        help="append a column of tick sizes for each field to be rounded")
+    parser.add_argument(
+        "-o", "--outfile",
+        metavar="OUTFILE",
+        dest="outfilepath_or_buffer",
+        help="filename to write the data to (default is stdout)")
+    parser.set_defaults(func="quantrocket.master._cli_round_to_tick_sizes")
