@@ -14,3 +14,9 @@
 
 class ImproperlyConfigured(Exception):
     pass
+
+class CannotConnectToHouston(Exception):
+    pass
+
+class UnavailableInsideJupyter(EnvironmentError):
+    pass
