@@ -13,6 +13,7 @@
 # limitations under the License.
 
 import sys
+import six
 from quantrocket.houston import houston
 from quantrocket.cli.utils.output import json_to_cli
 from quantrocket.cli.utils.stream import to_bytes
@@ -547,3 +548,154 @@ def _cli_load_or_show_rollrules(filename=None):
         return json_to_cli(load_rollrules_config, filename)
     else:
         return json_to_cli(get_rollrules_config)
+
+def fetch_calendar(exchanges=None):
+    """
+    Fetch upcoming trading hours for exchanges and save to securites master database.
+
+    Parameters
+    ----------
+    exchanges : list of str, optional
+        limit to these exchanges
+
+    Returns
+    -------
+    dict
+        status message
+    """
+    params = {}
+    if exchanges:
+        params["exchanges"] = exchanges
+
+    response = houston.post("/master/calendar", params=params)
+    houston.raise_for_status_with_json(response)
+    return response.json()
+
+def _cli_fetch_calendar(*args, **kwargs):
+    return json_to_cli(fetch_calendar, *args, **kwargs)
+
+def list_calendar_statuses(exchanges, sec_type=None, in_=None, ago=None):
+    """
+    Check whether exchanges are open or closed.
+
+    Parameters
+    ----------
+    exchanges : list of str, required
+        the exchange(s) to check
+
+    sec_type : str, optional
+        the security type, if needed to disambiguate for exchanges that
+        trade multiple security types. Possible choices: STK, FUT, CASH, OPT
+
+    in_ : str, optional
+        check whether exchanges will be open or closed at this point in the
+        future (use Pandas timedelta string, e.g. 2h or 30min or 1d)
+
+    ago : str, optional
+        check whether exchanges were open or closed this long ago
+        (use Pandas timedelta string, e.g. 2h or 30min or 1d)
+
+    Returns
+    -------
+    dict
+        exchange calendar status
+    """
+    params = {}
+    if exchanges:
+        params["exchanges"] = exchanges
+    if sec_type:
+        params["sec_type"] = sec_type
+    if in_:
+        params["in"] = in_
+    if ago:
+        params["ago"] = ago
+
+    response = houston.get("/master/calendar", params=params)
+    houston.raise_for_status_with_json(response)
+    return response.json()
+
+def _cli_list_calendar_statuses(*args, **kwargs):
+    return json_to_cli(list_calendar_statuses, *args, **kwargs)
+
+def _cli_isopen(*args, **kwargs):
+    statuses = list_calendar_statuses(*args, **kwargs)
+    is_open = all([
+        calendar["status"] == "open" for calendar in statuses.values()
+    ])
+    return '', int(not is_open)
+
+def _cli_isclosed(*args, **kwargs):
+    statuses = list_calendar_statuses(*args, **kwargs)
+    is_closed = all([
+        calendar["status"] == "closed" for calendar in statuses.values()
+    ])
+    return '', int(not is_closed)
+
+def round_to_tick_sizes(infilepath_or_buffer, round_fields,
+                        how=None, append_ticksize=False,
+                        outfilepath_or_buffer=None):
+    """
+    Round prices in a CSV file to valid tick sizes.
+
+    CSV should contain columns `ConId`, `Exchange`, and the columns to be rounded
+    (e.g. `LmtPrice`). Additional columns will be ignored and returned unchanged.
+
+    Parameters
+    ----------
+    infilepath_or_buffer : str or file-like object, required
+        CSV file with prices to be rounded (specify '-' to read file from stdin)
+
+    round_fields : list of str, required
+        columns to be rounded
+
+    how : str, optional
+        which direction to round to. Possible choices: 'up', 'down', 'nearest'
+        (default is 'nearest')
+
+    append_ticksize : bool
+        append a column of tick sizes for each field to be rounded (default False)
+
+    outfilepath_or_buffer : str or file-like object
+        filepath to write the data to, or file-like object (defaults to stdout)
+
+    Returns
+    -------
+    None
+    """
+
+    params = {}
+    if round_fields:
+        params["round_fields"] = round_fields
+    if how:
+        params["how"] = how
+    if append_ticksize:
+        params["append_ticksize"] = append_ticksize
+
+    url = "/master/ticksizes.csv"
+
+    if infilepath_or_buffer == "-":
+        # No-op if an empty file is passed on stdin
+        f = six.StringIO(sys.stdin.read())
+        if not f.getvalue():
+            return
+
+        response = houston.get(url, params=params, data=to_bytes(f))
+
+    elif infilepath_or_buffer and hasattr(infilepath_or_buffer, "read"):
+        if infilepath_or_buffer.seekable():
+            infilepath_or_buffer.seek(0)
+        response = houston.get(url, params=params, data=to_bytes(infilepath_or_buffer))
+
+    elif infilepath_or_buffer:
+        with open(infilepath_or_buffer, "rb") as f:
+            response = houston.get(url, params=params, data=f)
+    else:
+        raise ValueError("infilepath_or_buffer is required")
+
+    houston.raise_for_status_with_json(response)
+
+    filepath_or_buffer = outfilepath_or_buffer or sys.stdout
+    write_response_to_filepath_or_buffer(filepath_or_buffer, response)
+
+def _cli_round_to_tick_sizes(*args, **kwargs):
+    return json_to_cli(round_to_tick_sizes, *args, **kwargs)
\ No newline at end of file
